from django.db import models
from django.contrib.auth.models import User as DjangoUser

class Color(models.Model):
    """
    >>> red = Color.objects.create(name='red', hexcode='ff0000')
    >>> red
    <Color: red>
    """

    name        = models.CharField(max_length=20)
    hexcode     = models.CharField(max_length=8)

    def __str__(self):
        return self.name

    class Admin:
        search_fields = ['name', 'hexcode']
        list_display  = ['name', 'hexcode']

    class Meta:
        ordering = ['name']

class HiLite(models.Model):
    """
    >>> red = Color.objects.create(name='red', hexcode='ff0000')
    >>> banned_sites = HiLite.objects.create(name='banned sites', color=red)
    >>> banned_sites
    <HiLite: banned sites>
    """

    name        = models.CharField(max_length=32)
    comment     = models.TextField(null=True, blank=True)
    color       = models.ForeignKey(Color, null=True, blank=True)

    def __str__(self):
        return self.name

    class Admin:
        search_fields = ['name']
        list_display  = ['name', 'color']

    class Meta:
        ordering = ['name']

class Workstation(models.Model):
    """
    >>> workstation1 = Workstation.objects.create(ip_addr='172.17.2.7')
    >>> workstation2 = Workstation.objects.create(dns_name='pc017')
    >>> workstation3 = Workstation.objects.create(ip_addr='172.17.2.7', dns_name='pc017')
    >>> workstation1
    <Workstation: 172.17.2.7>
    >>> workstation2
    <Workstation: pc017>
    >>> workstation3
    <Workstation: pc017>
    """

    ip_addr     = models.IPAddressField(null=True, blank=True)
    dns_name    = models.CharField(max_length=80, null=True, blank=True)

    def __str__(self):
        if (self.dns_name):
            return str(self.dns_name)
        else:
            return self.ip_addr

    class Admin:
        search_fields = ['ip_addr', 'dns_name']
        list_display  = ['ip_addr', 'dns_name']

class User(models.Model):
    """
    >>> workstation = Workstation.objects.create(ip_addr='172.17.2.7', dns_name='pc017')
    >>> workstation
    <Workstation: pc017>
    >>> user = User.objects.create(username='Fred', flagged=False, exclude_from_database=False, exclude_from_reports=False)
    >>> user
    <User: Fred>
    >>> user.workstation.add(workstation)
    >>> user.workstation.all()
    [<Workstation: pc017>]
    """

    # The username will be the IP address if no other username is available
    username              = models.CharField(max_length=80)
    workstation           = models.ManyToManyField(Workstation, null=True, blank=True, filter_interface=True)
    flagged               = models.BooleanField()
    exclude_from_database = models.BooleanField()
    exclude_from_reports  = models.BooleanField()
    managed_by            = models.ForeignKey(DjangoUser, null=True, blank=True)

    def __str__(self):
        return self.username

    class Admin:
        search_fields = ['username']
        list_display  = ['username', 'managed_by', 'flagged', 'exclude_from_reports', 'exclude_from_database']
        list_filter   = ['flagged', 'managed_by', 'exclude_from_reports', 'exclude_from_database']

    class Meta:
        ordering = ['username']

class UserGroup(models.Model):
    """
    >>> usergroup = UserGroup.objects.create(name='testgroup', description='A test group')
    >>> usergroup
    <UserGroup: testgroup>
    >>> user = User.objects.create(username='Fred', flagged=False, exclude_from_database=False, exclude_from_reports=False)
    >>> user
    <User: Fred>
    >>> usergroup.users.add(user)
    >>> usergroup.users.all()
    [<User: Fred>]
    >>> usergroup.users.remove(user)
    >>> usergroup.users.all()
    []
    """

    name        = models.CharField(max_length=32)
    description = models.CharField(max_length=64, null=True, blank=True)
    users       = models.ManyToManyField(User, null=True, blank=True, filter_interface=True)

    #def save(self):
    #    if self.id:
    #        if (int(self.id) == 0): # Flagged Users group
    #            return
    #        else:
    #            super(UserGroup, self).save()
    #    else:
    #        super(UserGroup, self).save()

    #def delete(self):
    #    if (int(self.id) == 0): # Flagged Users group
    #        return
    #    else:
    #        super(UserGroup, self).delete()

    def __str__(self):
        return self.name

    class Admin:
        search_fields = ['name', 'description']
        list_display  = ['name', 'description']

    class Meta:
        ordering = ['name']

class Status(models.Model):
    """
    >>> status = Status.objects.create(code=0, description='No Response Received', category='Squid-specific')
    >>> status
    <Status: 0>
    """

    code        = models.PositiveSmallIntegerField()
    description = models.CharField(max_length=32)
    category    = models.CharField(max_length=16)

    def __str__(self):
        return str(self.code)

    class Admin:
        search_fields = ['code', 'description', 'category']
        list_display  = ['code', 'description', 'category']
        list_filter   = ['category']

    class Meta:
        ordering = ['code']
        verbose_name_plural = 'statuses'

class Tld(models.Model):
    """
    >>> tld = Tld.objects.create(name='co.uk', size=2)
    >>> tld
    <Tld: co.uk>
    """

    name = models.CharField(max_length=16)
    size = models.SmallIntegerField()

    def __str__(self):
        return self.name

class Website(models.Model):
    """
    >>> red = Color.objects.create(name='red', hexcode='ff0000')
    >>> banned_sites = HiLite.objects.create(name='banned sites', color=red)
    >>> banned_sites
    <HiLite: banned sites>
    >>> website = Website.objects.create(name='example.com', hilite=banned_sites)
    >>> website
    <Website: example.com>
    """

    name             = models.CharField(max_length=80)
    hilite           = models.ForeignKey(HiLite, null=True, blank=True)
    added            = models.DateTimeField(auto_now_add=True)
    last_visit       = models.DateTimeField(null=True, blank=True)
    top_level_site   = models.ForeignKey('self', null=True, blank=True)
    thumbnail        = models.CharField(max_length=128, null=True, blank=True)

    def __str__(self):
        return self.name

    class Admin:
        search_fields  = ['name', 'hilite']
        list_display   = ['name', 'added', 'hilite', 'thumbnail']
        date_hierarchy = 'last_visit'
        list_filter    = ['hilite']

    class Meta:
        ordering = ['name']

class Method(models.Model):
    """
    >>> method = Method.objects.create(name='CONNECT')
    >>> method
    <Method: CONNECT>
    """

    name        = models.CharField(max_length=80)

    def __str__(self):
        return self.name

    class Admin:
        search_fields = ['name']
        list_display  = ['name']

    class Meta:
        ordering = ['name']

class MimeType(models.Model):
    """
    >>> mimetype = MimeType.objects.create(type='application', description='vnd.acucobol', extension='acu')
    >>> mimetype
    <MimeType: application/vnd.acucobol>
    """

    type        = models.CharField(max_length=16)
    description = models.CharField(max_length=64)
    extension   = models.CharField(max_length=16)

    def __str__(self):
        return self.type + '/' + self.description

    class Admin:
        search_fields   = ['type', 'description', 'extension']
        list_display    = ['extension', 'description', 'type']
        list_filter     = ['type']

    class Meta:
        ordering        = ['extension']

class Weblog(models.Model):
    """
    >>> import datetime
    >>> user = User.objects.create(username='Fred', flagged=False, exclude_from_database=False, exclude_from_reports=False)
    >>> user
    <User: Fred>
    >>> method = Method.objects.create(name='CONNECT')
    >>> method
    <Method: CONNECT>
    >>> status = Status.objects.create(code=0, description='No Response Received', category='Squid-specific')
    >>> status
    <Status: 0>
    >>> mimetype = MimeType.objects.create(type='application', description='vnd.acucobol', extension='acu')
    >>> mimetype
    <MimeType: application/vnd.acucobol>
    >>> red = Color.objects.create(name='red', hexcode='ff0000')
    >>> banned_sites = HiLite.objects.create(name='banned sites', color=red)
    >>> banned_sites
    <HiLite: banned sites>
    >>> website = Website.objects.create(name='example.com', hilite=banned_sites)
    >>> website
    <Website: example.com>
    >>> weblog = Weblog.objects.create(user=user, log_datetime=datetime.datetime.now(), method=method, url='http://www.example.com/dir/thingy.acu', status=status, size=2079, website=website, mimetype=mimetype, flagged=True)
    >>> weblog
    <Weblog: http://www.example.com/dir/thingy.acu>
    """

    user         = models.ForeignKey(User)
    log_datetime = models.DateTimeField()
    method       = models.ForeignKey(Method)
    url          = models.TextField(max_length=2048)
    status       = models.ForeignKey(Status)
    size         = models.PositiveIntegerField()
    website      = models.ForeignKey(Website)
    mimetype     = models.ForeignKey(MimeType, blank=True, null=True)
    flagged      = models.BooleanField()

    def __str__(self):
        return str(self.url)

    class Admin:
        search_fields  = ['user', 'url', 'website']
        list_display   = ['user', 'log_datetime', 'method', 'url', 'status', 'size', 'website']
        list_filter    = ['method', 'status']
        date_hierarchy = 'log_datetime'

    class Meta:
        ordering = ['log_datetime']


class UserMapping(models.Model):
    """
    >>> workstation = Workstation.objects.create(ip_addr='172.17.2.7')
    >>> workstation
    <Workstation: 172.17.2.7>
    >>> user = User.objects.create(username='Fred', flagged=False, exclude_from_database=False, exclude_from_reports=False)
    >>> user
    <User: Fred>
    >>> usermapping = UserMapping.objects.create(workstation=workstation, user=user)
    >>> usermapping
    <UserMapping: 172.17.2.7 -> Fred>
    """

    workstation = models.ForeignKey(Workstation, unique=True)
    user        = models.ForeignKey(User)

    def __str__(self):
        return str(self.workstation) + ' -> ' + str(self.user)

    class Admin:
        search_fields = ['workstation', 'user']
        list_display  = ['workstation', 'user']

    class Meta:
        ordering = ['workstation']


class ViewPermissions(models.Model):
    """
    >>> viewpermission = ViewPermissions.objects.create(name='runtests')
    >>> viewpermission
    <ViewPermissions: runtests>
    """

    name = models.CharField(max_length=32)

    def __str__(self):
        return self.name

class UserProfile(models.Model):
    '''
    >>> viewpermission = ViewPermissions.objects.create(name='runtests')
    >>> viewpermission
    <ViewPermissions: runtests>
    >>> pw = DjangoUser.objects.make_random_password(length=10, allowed_chars='abcdefghjkmnpqrstuvwxyzABCDEFGHJKLMNPQRSTUVWXYZ23456789')
    >>> rem_user = 'Test user'
    >>> django_user = DjangoUser.objects.create_user(username=rem_user, email=rem_user + '@example.com', password=pw)
    >>> django_user
    <User: Test user>
    >>> django_user.is_staff = True
    >>> django_user.is_superuser = True
    >>> view_perms = ViewPermissions.objects.all()
    >>> user_profile, created = UserProfile.objects.get_or_create(user=django_user)
    >>> user_profile
    <UserProfile: Test user>
    >>> for vp in view_perms: user_profile.view_permissions.add(vp)
    >>> user_profile.delete()
    >>> django_user.delete()
    '''

    user                = models.ForeignKey(DjangoUser, unique=True)
    notes               = models.TextField(blank=True, null=True)
    view_permissions    = models.ManyToManyField(ViewPermissions, blank=True, null=True, filter_interface=True)

    def __str__(self):
        return str(self.user)

    class Admin:
        search_fields = ['user']
        list_display  = ['user']

    class Meta:
        ordering = ['user']

