from django.db import models
from django.contrib.auth.models import User as DjangoUser
from dateutil.parser import *
import datetime


SEVERITY = ( (1, 'Fatal'), (2, 'Warning'), (3, 'Information') )


class GeneralSetting(models.Model):
    upload_logfile  = models.FileField(upload_to='logs/%Y/%m/%d/%s', blank=True, null=True, \
                      help_text='Upload a Squid or Dan\'s Guardian logfile. \
                      Or enter a value in the "logfile" field for live data acquisition.')
    start_date      = models.DateTimeField(null=True, blank=True)
    end_date        = models.DateTimeField(null=True, blank=True)
    row_limit       = models.IntegerField(default=200, help_text='Maximum number of rows to return in any query (0 = all rows).')
    use_thumbnails  = models.BooleanField()
    owner           = models.ForeignKey(DjangoUser)

    def __str__(self):
        return str(self.owner)

    def save(self):
        if not self.start_date:
            self.start_date = datetime.datetime.now() - datetime.timedelta(hours=24)
        if not self.end_date:
            self.end_date = datetime.datetime.now()
        super(GeneralSetting, self).save()

    class Admin:
        list_display   = ['owner', 'start_date', 'end_date', 'row_limit', 'upload_logfile']


class Message(models.Model):
    date_added  = models.DateTimeField()
    text        = models.TextField(max_length=512)
    severity    = models.SmallIntegerField(choices=SEVERITY)
    source      = models.TextField(max_length=256, null=True, blank=True)

    def __str__(self):
        return self.text

    class Admin:
        list_display    = ['date_added', 'text', 'severity', 'source']
        search_fields   = ['text', 'source']
        list_filter     = ['severity']
        date_hierarchy  = 'date_added'

    class Meta:
        ordering = ['date_added']


class FileLock(models.Model):
    filename    = models.TextField(max_length=256)
    started     = models.DateTimeField(null=True, blank=True)
    finished    = models.DateTimeField(null=True, blank=True)

    def __str__(self):
        return self.filename

    class Admin:
        list_display    = ['filename', 'started', 'finished']
        search_fields   = ['filename']
        date_hierarchy  = 'started'

    class Meta:
        ordering = ['started']


UNITS = (('m', 'Minutes'), ('h', 'Hours'), ('d', 'Days'), ('w', 'Weeks'))


class TimeSlice(models.Model):
    number  = models.IntegerField()
    units   = models.CharField(max_length=1, choices=UNITS)

    def __str__(self):
        return str(self.number) + '_' + self.units

    class Admin:
        list_display = ['number', 'units']





